/* Main header for OperStats

   OperStats - Copyright (c) 2000 Darcy Grexton
   Contact: cygnus@uniserve.ca, skold @ HabberNet, ICQ #6285178

   This program is free but copyrighted software; see the file COPYING
   for details.
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <setjmp.h>
#include <sys/types.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <grp.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include "config.h"
#include "version.h"
#include "sysconf.h"
#include "response.h"

/* true/false stuff */
typedef enum {ERROR = -1,FALSE, TRUE} Boolean_T;

/* Flags */
#define RUN_DEBUG	0x00000001	/* Debug mode enabled	*/
#define RUN_LIVE	0x00000002	/* Don't fork		*/
#define RUN_QUITTING	0x00000004	/* Shutting down	*/
#define RUN_STARTED	0x00000008	/* Just Started		*/
#define RUN_TERMINATING	0x00000010	/* Terminating		*/
#define RUN_SIGTERM	0x00000020	/* Sigterm		*/
#define RUN_SAVEDBS	0x00000040	/* Save DBs		*/

/* User Flags */
#define UF_SRA		0x00000001	/* Is an SRA		*/

/* User Modes - Only modes we care about */
#define UMODE_o		0x00000001	/* Opered		*/
#define UMODE_i		0x00000002	/* Invisible		*/
#define UMODE_s		0x00000004	/* See server notices	*/
#define UMODE_w		0x00000008	/* See wallops		*/
#define UMODE_g		0x00000010	/* See globops		*/
#define UMODE_r		0x00000020	/* Registered nick	*/

/* Channel Modes */
#define CMODE_i		0x00000001	/* Invite only		*/
#define CMODE_m		0x00000002	/* Moderated		*/
#define CMODE_n		0x00000004	/* No external messages	*/
#define CMODE_p		0x00000008	/* Private		*/
#define CMODE_s		0x00000010	/* Secret		*/
#define CMODE_t		0x00000020	/* Only ops change topic*/
#define CMODE_k		0x00000040	/* Keyed		*/
#define CMODE_l		0x00000080	/* Limited to # users	*/
#define CMODE_R		0x00000100	/* Registered nicks only*/
#define CMODE_r		0x00000200	/* Registered channel	*/
#define CMODE_c		0x00000400	/* No colors		*/

/* Channel hash */
#define CHASH(chan) ((chan)[1] ? ((chan)[1]&31)<<5 | ((chan)[2]&31) : 0)

/* Nick hash */
#define NHASH(nick) (((nick)[0]&31)<<5 | ((nick)[1]&31))

/* Server hash */
#define SHASH(server) (((server)[0]&31)<<5 | ((server)[1]&31))

/* AKill hash */
#define AHASH(akill) (((akill)[0]&31)<<5 | ((akill)[1]&31))

/* Structure for all our stuff */
struct me
{
    char uplink[HOSTLEN];	/* Address we connect to	*/
    char bindhost[HOSTLEN];	/* Address to bind to		*/
    int port;			/* Port we do it on		*/
    int bindport;		/* Port to bind to		*/
    char pass[PASSWDLEN];	/* Password we use for the link	*/
    char name[HOSTLEN];		/* Our server name		*/
    char desc[REALLEN];		/* Our description		*/
    char ruplink[HOSTLEN];	/* Real name of our uplink	*/
    time_t since;		/* When we started		*/
    time_t reset;		/* When we last reset		*/
} me;

struct os
{
    char user[USERLEN];		/* OperServ's username		*/
    char host[HOSTLEN];		/* OperServ's hostname		*/
    char mode[12];		/* OperServ's modes		*/
    char real[REALLEN];		/* OperServ's realname		*/
} os;

struct ss
{
    char user[USERLEN];		/* StatServ's username		*/
    char host[HOSTLEN];		/* StatServ's hostname		*/
    char mode[12];		/* StatServ's modes		*/
    char real[REALLEN];		/* StatServ's realname		*/
} ss;

struct gn
{
    char user[USERLEN];		/* GlobalNoticer's username	*/
    char host[HOSTLEN];		/* GlobalNoticer's hostname	*/
    char mode[12];		/* GlobalNoticer's modes	*/
    char real[REALLEN];		/* GlobalNoticer's realname	*/
} gn;

typedef struct user_ User;
typedef struct channel_ Channel;
typedef struct sra_ SRA;
typedef struct tld_ TLD;
typedef struct msg_ MSG;
typedef struct servstat_ ServStat;
typedef struct akill_ AutoKill;

struct user_ {
    User *next, *prev;
    char nick[NICKLEN];
    char *user;
    char *host;
    char *real;
    char *server;
    time_t signon;
    int32 mode;
    int32 flags;
    struct u_chanlist {
	struct u_chanlist *next, *prev;
	Channel *chan;
    } *chans;
};

struct channel_ {
    Channel *next, *prev;
    char name[CHANNELLEN];
    time_t creation_time;
    char *topic;
    char topic_setter[NICKLEN];
    time_t topic_time;
    int32 mode;
    int32 limit;
    char *key;
    int32 bancount, bansize;
    char **bans;
    struct c_userlist {
	struct c_userlist *next, *prev;
	User *user;
    } *users, *chanops, *voices;
};

struct sra_ {
    SRA *next, *prev;
    char *nick;
    char *pass;
};

struct tld_ {
    TLD *next, *prev;
    char *name;
    char *desc;
};

struct msg_ {
    MSG *next, *prev;
    char *text;
    char *setter;
    time_t set;
};

typedef struct {
    char *server;
    char *uplink;
    char *desc;
    time_t connect;
    time_t lastping;
    time_t lastpong;
    int hops;
    int lag;
} Servers;

struct servstat_ {
    ServStat *next, *prev;
    char *name;
    char *version;
    char *flags;
    time_t statssince;
    time_t lastsplit;
    int is_split;
    int splittime;
    int splits;
};

struct akill_ {
    AutoKill *next, *prev;
    char *mask;
    char *reason;
    char *setter;
    int realname;
    time_t set;
    time_t expires;
};

typedef struct {
    const char *name;
    void (*func) (char *source, int ac, char **av);
} Message;

extern Message messages[];
extern Message *find_message (const char *name);

typedef struct {   
    char *accept;
    short access;
    void (*process) (const char *source);
} Hash;

typedef struct {
    char *accept;
    short access;
    const char *(*process);
} Hash_HELP;

extern Hash *get_hash (const char *whoami, const char *source, const char *cmd, Hash *hash_table); 
extern Hash_HELP *get_help_hash (const char *whoami, const char *source, const char *cmd, Hash_HELP *hash_table);

#define H_NONE  0
#define H_IRCOP 1
#define H_SRA   2

/* This needs to be down here so the stuff in operstats.h it uses works */
#include "extern.h"
