/* Config file routines

   OperStats - Copyright (c) 2000 Darcy Grexton
   Contact: cygnus@uniserve.ca, skold @ HabberNet, ICQ #6285178

   This program is free but copyrighted software; see the file COPYING
   for details.
 */

#include "../inc/operstats.h"

SRA *sralist = NULL;

/* Some vars */
char s_OperServ[NICKLEN], s_StatServ[NICKLEN], s_GlobalNoticer[NICKLEN];
char tlds[64], snoopchan[32], def_akill_reason[256];
Boolean_T operserv_on = FALSE, statserv_on = FALSE, globalnoticer_on = FALSE;
Boolean_T globalonakill_on = FALSE, stamplogons_on = FALSE;
int update_timeout, ping_delay, nonwildreq, def_akill_time;

/* Add a new SRA into the sra list. */ 
static SRA * new_sra (const char *nick, const char *pass)
{
    SRA *sra;

    if (sra = findsra (nick)) /* Don't want dupes, rehash */
    {
	sra->pass = sstrdup (pass);
	return;
    }

    sra = scalloc (sizeof (SRA), 1);

    sra->nick = sstrdup (nick);
    sra->pass = sstrdup (pass);

    sra->next = sralist;

    if (sra->next)
        sra->next->prev = sra;

    sralist = sra;
}

/* Load and parse the conf file */
void load_conf (void)
{
    FILE *f = fopen (CONFIGFILE, "r");
    char *item, *s, dBuf[BUFSIZE];

    if (!f)
    {
	log (ERR_NO_CONFIG);
	exit (1);
    }

    while (fgets (dBuf, 2047, f))
    {
	item = strtok (dBuf, " ");
	strip (item);

	if (*item == '#' || *item == '\t' || *item == ' ')
	    continue;

	if (!stricmp (item, "Info"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s && strchr (s, '.'))
		memcpy (me.name, s, sizeof (me.name));
	    else
		fatal ("Error in config: Info is not set properly.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (me.desc, s, sizeof (me.desc));
	    else
		fatal ("Error in config: Info is not set properly.");
	}

	if (!stricmp (item, "BindTo"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.bindhost, s, sizeof (me.bindhost));
	    else
		fatal ("Error in config: BindTo is not set properly.");

	    s = strtok (NULL, " ");   
	    strip (s);

	    if (s)
		me.bindport = atoi (s);
	    else  
		fatal ("Error in config: BindTo is not set properly.");
	}

	if (!stricmp (item, "Connect"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.uplink, s, sizeof (me.uplink));
	    else
		fatal ("Error in config: Connect is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.pass, s, sizeof (me.pass));
	    else
		fatal ("Error in config: Connect is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.ruplink, s, sizeof (me.ruplink));
	    else
		fatal ("Error in config: Connect is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		me.port = atoi (s);
	    else
		fatal ("Error in config: Connect is not set properly.");
	}

	if (!stricmp (item, "OperServ"))
	{
	    operserv_on = TRUE;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (s_OperServ, s, sizeof (s_OperServ));
	    else
		fatal ("Error in config: OperServ is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (os.user, s, sizeof (os.user));
	    else
		fatal ("Error in config: OperServ is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (os.host, s, sizeof (os.host));
	    else
		fatal ("Error in config: OperServ is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
	    {
		if (!strchr (s, '+'))
		    fatal ("Error in config: OperServ: You need a + for modes!");

		memcpy (os.mode, s, sizeof (os.mode));
	    }
	    else
		fatal ("Error in config: OperServ is not set properly.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (os.real, s, sizeof (os.real));
	    else
		fatal ("Error in config: OperServ is not set properly.");
	}

	if (!stricmp (item, "StatServ"))
	{
	    statserv_on = TRUE;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (s_StatServ, s, sizeof (s_StatServ));
	    else
		fatal ("Error in config: StatServ is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (ss.user, s, sizeof (ss.user));
	    else
		fatal ("Error in config: StatServ is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (ss.host, s, sizeof (ss.host));
	    else
		fatal ("Error in config: StatServ is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
	    {
		if (!strchr (s, '+'))
		    fatal ("Error in config: StatServ: You need a + for modes!");

		memcpy (ss.mode, s, sizeof (ss.mode));
	    }
	    else
		fatal ("Error in config: StatServ is not set properly.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (ss.real, s, sizeof (ss.real));
	    else
		fatal ("Error in config: StatServ is not set properly.");
	}

	if (!stricmp (item, "GlobalNoticer"))
	{
	    globalnoticer_on = TRUE;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (s_GlobalNoticer, s, sizeof (s_GlobalNoticer));
	    else
		fatal ("Error in config: GlobalNoticer is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (gn.user, s, sizeof (gn.user));
	    else
		fatal ("Error in config: GlobalNoticer is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (gn.host, s, sizeof (gn.host));
	    else
		fatal ("Error in config: GlobalNoticer is not set properly.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
	    {
		if (!strchr (s, '+'))
		    fatal ("Error in config: GlobalNoticer: You need a + for modes!"); 

		memcpy (gn.mode, s, sizeof (gn.mode));
	    }
	    else
		fatal ("Error in config: GlobalNoticer is not set properly.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (gn.real, s, sizeof (gn.real));
	    else
		fatal ("Error in config: GlobalNoticer is not set properly.");
        }

	if (!stricmp (item, "SRA"))
	{
	    char *nick, *pass;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		nick = s;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		pass = s;

	    if (!pass)
		fatal ("Error in config: Missing password for SRA %s", nick);
	    else
		new_sra (nick, pass);
        }

	if (!stricmp (item, "SyncTime"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		update_timeout = dotime (s);  
	    else
		fatal ("Error in config: SyncTime is not set properly.");
	}

	if (!stricmp (item, "PingDelay"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		ping_delay = dotime (s);
	    else
		fatal ("Error in config: PingDelay is not set properly.");
	}

	if (!stricmp (item, "SnoopChan"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (snoopchan, s, sizeof (snoopchan));
	    else
		fatal ("Error in config: SnoopChan is not set properly.");
	}

	if (!stricmp (item, "TLD"))
	{  
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (tlds, s, sizeof (tlds));
	    else
		fatal ("Error in config: TLD is not set properly.");
        }

	if (!stricmp (item, "AKillExpire"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		def_akill_time = dotime (s);
	    else
		fatal ("Error in config: AKillExpire is not set properly.");
	}

	if (!stricmp (item, "NonWildCards"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		nonwildreq = atol (s);
	    else
		fatal ("Error in config: NonWildCards is not set properly.");
	}

	if (!stricmp (item, "GlobalOnAKill"))
	    globalonakill_on = TRUE;

	if (!stricmp (item, "StampLogons"))
	    stamplogons_on = TRUE;

	if (!stricmp (item, "DefaultAKillReason"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (def_akill_reason, s, sizeof (def_akill_reason));
	    else
		fatal ("Error in config: DefaultAKillReason is not set properly.");
	}
    }
    fclose (f);
}

/* Sanity checks on the config options */
int check_conf (void)
{
    if (operserv_on == FALSE)
	fatal ("Error in config: OperServ cannot be disabled.");

    if (!update_timeout)
	update_timeout = 1200;

    if (!ping_delay)
	ping_delay = 60;

    if (!def_akill_time)
	def_akill_time = 604800; /* One week */

    if (!nonwildreq)
	nonwildreq = 5;

    if (update_timeout < 60 * 5)
	printf ("Warning: Setting SyncTime less than 5 minutes is unwise!\n");

    if (!strlen (tlds) && (globalnoticer_on == TRUE))
    {
        printf ("Error in config: TLD is not set properly. GlobalNoticer disabled.\n");
	globalnoticer_on = FALSE;
    }

    if (!strlen (def_akill_reason))
	memcpy (def_akill_reason, "You are banned.", sizeof (def_akill_reason));
}
